// This file is automatically generated from ./dns_tcp.pac.

#include "../src/dns_tcp_pac.h"

namespace binpac {


namespace DNS_on_TCP {
ContextDNS_on_TCP::ContextDNS_on_TCP(DNS_TCP_Conn * connection, DNS_TCP_Flow * flow, FlowBuffer * flow_buffer)
	{
	connection_ = connection;
	flow_ = flow;
	flow_buffer_ = flow_buffer;
	}

ContextDNS_on_TCP::~ContextDNS_on_TCP()
	{
	}

DNS_TCP_PDU::DNS_TCP_PDU(bool is_orig)
	{
	msglen_ = 0;
	is_orig_ = is_orig;
	byteorder_ = bigendian;
	buffering_state_ = 0;
	}

DNS_TCP_PDU::~DNS_TCP_PDU()
	{
	msg_.free();
	}

bool DNS_TCP_PDU::ParseBuffer(flow_buffer_t t_flow_buffer, ContextDNS_on_TCP * t_context)
	{
	bool t_val_parsing_complete;
	t_val_parsing_complete = false;
	const_byteptr t_begin_of_data = t_flow_buffer->begin();
	const_byteptr t_end_of_data = t_flow_buffer->end();
	switch ( buffering_state_ )
		{
		case 0:
			if ( buffering_state_ == 0 )
				{
				t_flow_buffer->NewFrame(2, false);
				buffering_state_ = 1;
				}
			buffering_state_ = 1;
			break;
		case 1:
			{
			buffering_state_ = 2;
			// Checking out-of-bound for "DNS_TCP_PDU:msglen"
			if ( t_begin_of_data + (2) > t_end_of_data )
				{
				// Handle out-of-bound condition
				throw ExceptionOutOfBound("DNS_TCP_PDU:msglen",
					(0) + (2), 
					(t_end_of_data) - (t_begin_of_data));
				}
			// Parse "msglen"
			msglen_ = FixByteOrder(byteorder(), *((uint16 const *) (t_begin_of_data)));
			// Evaluate 'let' and 'withinput' fields
			t_flow_buffer->GrowFrame(2 + msglen());
			}
			break;
		case 2:
			BINPAC_ASSERT(t_flow_buffer->ready());
			if ( t_flow_buffer->ready() )
				{
				
				// Parse "msg"
				int t_msg__size;
				t_msg__size = msglen();
				// Checking out-of-bound for "DNS_TCP_PDU:msg"
				if ( (t_begin_of_data + 2) + (t_msg__size) > t_end_of_data )
					{
					// Handle out-of-bound condition
					throw ExceptionOutOfBound("DNS_TCP_PDU:msg",
						(2) + (t_msg__size), 
						(t_end_of_data) - (t_begin_of_data));
					}
				{
				// Setting t_end_of_data with &length
				const_byteptr t_end_of_data = (t_begin_of_data + 2) + t_msg__size;
				int t_msg_string_length;
				t_msg_string_length = msglen();
				// check for negative sizes
				if ( t_msg_string_length < 0 )
				throw ExceptionInvalidStringLength("./dns_tcp.pac:16", t_msg_string_length);
				msg_.init((t_begin_of_data + 2), t_msg_string_length);
				// Evaluate 'let' and 'withinput' fields
				}
				
				t_val_parsing_complete = true;
				if ( t_val_parsing_complete )
					{
					// Evaluate 'let' and 'withinput' fields
					deliver_ = t_context->connection()->deliver_dns_message(is_orig(), msg());
					}
				BINPAC_ASSERT(t_val_parsing_complete);
				buffering_state_ = 0;
				}
			break;
		default:
			BINPAC_ASSERT(buffering_state_ <= 2);
			break;
		}
	return t_val_parsing_complete;
	}

DNS_TCP_Conn::DNS_TCP_Conn(BroAnalyzer const & bro_analyzer)
	{
	upflow_ = new DNS_TCP_Flow(this, true);
	downflow_ = new DNS_TCP_Flow(this, false);
	bro_analyzer_ = bro_analyzer;

		abstract_dns_connection_ = new DNS::DNS_Conn(bro_analyzer);
	
	}

DNS_TCP_Conn::~DNS_TCP_Conn()
	{

		delete abstract_dns_connection_;
		abstract_dns_connection_ = 0;
	
	delete upflow_;
	upflow_ = 0;
	delete downflow_;
	downflow_ = 0;
	}

void DNS_TCP_Conn::NewData(bool is_orig, const_byteptr begin, const_byteptr end)
	{
	if ( is_orig )
		upflow_->NewData(begin, end);
	else
		downflow_->NewData(begin, end);
	}

void DNS_TCP_Conn::NewGap(bool is_orig, int gap_length)
	{
	if ( is_orig )
		upflow_->NewGap(gap_length);
	else
		downflow_->NewGap(gap_length);
	}

void DNS_TCP_Conn::FlowEOF(bool is_orig)
	{
	if ( is_orig )
		upflow_->FlowEOF();
	else
		downflow_->FlowEOF();
	}

bool DNS_TCP_Conn::deliver_dns_message(bool is_orig, const_bytestring const & msg)
	{

		abstract_dns_connection_->NewData(is_orig, msg.begin(), msg.end());
		return true;
		
	}

DNS_TCP_Flow::DNS_TCP_Flow(DNS_TCP_Conn * connection, bool is_orig)
	{
	flow_buffer_ = 0;
	connection_ = connection;
	is_orig_ = is_orig;
	dataunit_ = 0;
	context_ = 0;
	flow_buffer_ = new FlowBuffer();
	}

DNS_TCP_Flow::~DNS_TCP_Flow()
	{
	delete dataunit_;
	dataunit_ = 0;
	delete context_;
	context_ = 0;
	delete flow_buffer_;
	flow_buffer_ = 0;
	}

void DNS_TCP_Flow::NewData(const_byteptr t_begin_of_data, const_byteptr t_end_of_data)
	{
	try
		{
		flow_buffer_->NewData(t_begin_of_data, t_end_of_data);
		while ( flow_buffer_->data_available() && 
			( !flow_buffer_->have_pending_request() || flow_buffer_->ready() ) )
			{
			if ( ! dataunit_ )
				{
				BINPAC_ASSERT(!context_);
				dataunit_ = new DNS_TCP_PDU(is_orig());
				flow_buffer_->NewFrame(0, false);
				context_ = new ContextDNS_on_TCP(connection(), this, flow_buffer_);
				}
			bool t_dataunit_parsing_complete;
			t_dataunit_parsing_complete = false;
			while ( ! t_dataunit_parsing_complete && flow_buffer_->ready() )
				{
				const_byteptr t_begin_of_data = flow_buffer_->begin();
				const_byteptr t_end_of_data = flow_buffer_->end();
				t_dataunit_parsing_complete = dataunit_->ParseBuffer(flow_buffer_, context_);
				if ( t_dataunit_parsing_complete )
					{
					// Evaluate 'let' and 'withinput' fields
					}
				}
			if ( t_dataunit_parsing_complete )
				{
				// Clean up the flow unit after parsing
				delete dataunit_;
				dataunit_ = 0;
				delete context_;
				context_ = 0;
				}
			else
				{
				// Resume upon next input segment
				BINPAC_ASSERT(!flow_buffer_->ready());
				break;
				}
			}
		}
	catch ( Exception const &e )
		{
		delete dataunit_;
		dataunit_ = 0;
		delete context_;
		context_ = 0;
		flow_buffer_->DiscardData();
		throw e;
		}
	}

void DNS_TCP_Flow::NewGap(int gap_length)
	{
	flow_buffer_->NewGap(gap_length);
	}
void DNS_TCP_Flow::FlowEOF()
	{
	flow_buffer_->set_eof();
	NewData(0, 0);
	}
} // namespace DNS_on_TCP
}  // namespace binpac
