// This file is automatically generated from ./ssl-record-layer.pac.

#ifndef __ssl_record_layer_pac_h
#define __ssl_record_layer_pac_h

#include <vector>

#include "binpac.h"


#include "binpac_bro.h"


#include <string>
using std::string;


#include "ssl_pac.h"
using binpac::SSL::SSLAnalyzer;

namespace binpac {

namespace SSLRecordLayer {
class ContextSSLRecordLayer;
enum ContentType {
	CHANGE_CIPHER_SPEC = 20,
	ALERT = 21,
	HANDSHAKE = 22,
	APPLICATION_DATA = 23,
	V2_ERROR = 300,
	V2_CLIENT_HELLO = 301,
	V2_CLIENT_MASTER_KEY = 302,
	V2_SERVER_HELLO = 304,
	UNKNOWN_OR_V2_ENCRYPTED = 400,
};
enum SSLVersions {
	UNKNOWN_VERSION = 0,
	SSLv20 = 2,
	SSLv30 = 768,
	TLSv10 = 769,
	TLSv11 = 770,
};
class SSLPDU;
class SSLRecordLayerAnalyzer;
class SSLRecordLayerFlow;
} // namespace SSLRecordLayer

int bytestring_to_int(const_bytestring const & s, int base);
double bytestring_to_double(const_bytestring const & s);
int bytestring_casecmp(const_bytestring const & s1, const_charptr const & s2);
bool bytestring_caseprefix(const_bytestring const & s1, const_charptr const & s2);
double network_time();
namespace SSLRecordLayer {

class ContextSSLRecordLayer
{
public:
	ContextSSLRecordLayer(SSLRecordLayerAnalyzer * analyzer, SSLRecordLayerFlow * flow, FlowBuffer * flow_buffer);
	~ContextSSLRecordLayer();
	
	// Member access functions
	SSLRecordLayerAnalyzer * analyzer() const { return analyzer_; }
	SSLRecordLayerFlow * flow() const { return flow_; }
	FlowBuffer * flow_buffer() const { return flow_buffer_; }
	
protected:
	SSLRecordLayerAnalyzer * analyzer_;
	SSLRecordLayerFlow * flow_;
	FlowBuffer * flow_buffer_;
};


class SSLPDU
{
public:
	SSLPDU();
	~SSLPDU();
	int initial_buffer_length() const { return 5; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSLRecordLayer * t_context);
	
	// Member access functions
	uint8 head0() const { return head0_; }
	uint8 head1() const { return head1_; }
	uint8 head2() const { return head2_; }
	uint8 head3() const { return head3_; }
	uint8 head4() const { return head4_; }
	bytestring const & fragment() const { return fragment_; }
	int byteorder() const { return byteorder_; }
	int version() const { return version_; }
	int length() const { return length_; }
	bool fw() const { return fw_; }
	
protected:
	uint8 head0_;
	uint8 head1_;
	uint8 head2_;
	uint8 head3_;
	uint8 head4_;
	bytestring fragment_;
	int byteorder_;
	int version_;
	int length_;
	bool fw_;
	int buffering_state_;
};


class SSLRecordLayerAnalyzer : public binpac::ConnectionAnalyzer
{
public:
	SSLRecordLayerAnalyzer();
	~SSLRecordLayerAnalyzer();
	
	// Member access functions
	SSLRecordLayerFlow * upflow() const { return upflow_; }
	SSLRecordLayerFlow * downflow() const { return downflow_; }
	
	void NewData(bool is_orig, const_byteptr begin, const_byteptr end);
	void NewGap(bool is_orig, int gap_length);
	void FlowEOF(bool is_orig);
	
	// Functions
	void set_ssl_analyzer(SSLAnalyzer * a);
	int ssl_version();
	int record_length();
	int determine_ssl_version(uint8 head0, uint8 head1, uint8 head2);
	bool forward_record(const_bytestring const & fragment, int type, uint16 version, bool is_orig);
	bool forward_v2_record(uint8 b1, uint8 b2, uint8 b3, const_bytestring const & fragment, bool is_orig);
	
protected:
	
	// Additional members

		SSLAnalyzer* ssl_analyzer_;

		int ssl_version_;
		int record_length_;
	
	SSLRecordLayerFlow * upflow_;
	SSLRecordLayerFlow * downflow_;
};


class SSLRecordLayerFlow : public binpac::FlowAnalyzer
{
public:
	SSLRecordLayerFlow(SSLRecordLayerAnalyzer * connection, bool is_orig);
	~SSLRecordLayerFlow();
	
	// Member access functions
	SSLRecordLayerAnalyzer * connection() const { return connection_; }
	bool is_orig() const { return is_orig_; }
	
	void NewData(const_byteptr t_begin_of_data, const_byteptr t_end_of_data);
	void NewGap(int gap_length);
	void FlowEOF();
	
	// Functions
	bool discard_data();
	
protected:
	SSLPDU * dataunit_;
	ContextSSLRecordLayer * context_;
	FlowBuffer * flow_buffer_;
	SSLRecordLayerAnalyzer * connection_;
	bool is_orig_;
};

} // namespace SSLRecordLayer
}  // namespace binpac
#endif /* __ssl_record_layer_pac_h */
