// This file is automatically generated from ./ssl.pac.

#ifndef __ssl_pac_h
#define __ssl_pac_h

#include <vector>

#include "binpac.h"


#include "binpac_bro.h"


#include <string>
using std::string;


#include <vector>
#include <algorithm>
#include <iostream>
#include <iterator>

#include "util.h"

#ifdef USE_OPENSSL
#include <openssl/x509.h>
#include <openssl/x509_vfy.h>
#include "X509.h"
#endif

namespace binpac {

namespace SSL {
class ContextSSL;
enum ContentType {
	CHANGE_CIPHER_SPEC = 20,
	ALERT = 21,
	HANDSHAKE = 22,
	APPLICATION_DATA = 23,
	V2_ERROR = 300,
	V2_CLIENT_HELLO = 301,
	V2_CLIENT_MASTER_KEY = 302,
	V2_SERVER_HELLO = 304,
	UNKNOWN_OR_V2_ENCRYPTED = 400,
};
enum SSLVersions {
	UNKNOWN_VERSION = 0,
	SSLv20 = 2,
	SSLv30 = 768,
	TLSv10 = 769,
	TLSv11 = 770,
};
class uint24;
enum AnalyzerState {
	STATE_INITIAL,
	STATE_CLIENT_HELLO_RCVD,
	STATE_IN_SERVER_HELLO,
	STATE_SERVER_HELLO_DONE,
	STATE_CLIENT_CERT,
	STATE_CLIENT_KEY_WITH_CERT,
	STATE_CLIENT_KEY_NO_CERT,
	STATE_CLIENT_CERT_VERIFIED,
	STATE_CLIENT_ENCRYPTED,
	STATE_CLIENT_FINISHED,
	STATE_ABBREV_SERVER_ENCRYPTED,
	STATE_ABBREV_SERVER_FINISHED,
	STATE_COMM_ENCRYPTED,
	STATE_CONN_ESTABLISHED,
	STATE_V2_CL_MASTER_KEY_EXPECTED,
	STATE_TRACK_LOST,
	STATE_ANY,
};
enum HandshakeType {
	HELLO_REQUEST = 0,
	CLIENT_HELLO = 1,
	SERVER_HELLO = 2,
	CERTIFICATE = 11,
	SERVER_KEY_EXCHANGE = 12,
	CERTIFICATE_REQUEST = 13,
	SERVER_HELLO_DONE = 14,
	CERTIFICATE_VERIFY = 15,
	CLIENT_KEY_EXCHANGE = 16,
	FINISHED = 20,
};
class ChangeCipherSpec;
class Alert;
class V2Error;
class ApplicationData;
class HelloRequest;
class ClientHello;
class V2ClientHello;
class ServerHello;
class V2ServerHello;
class X509Certificate;
class CertificateList;
class Certificate;
class ServerKeyExchange;
class CertificateRequest;
class ServerHelloDone;
class ClientKeyExchange;
class V2ClientMasterKey;
class CertificateVerify;
class UnknownHandshake;
class Handshake;
class UnknownRecord;
class PlaintextRecord;
class CiphertextRecord;
class SSLPDU;
class SSLAnalyzer;
class SSLFlow;
} // namespace SSL

int bytestring_to_int(const_bytestring const & s, int base);
double bytestring_to_double(const_bytestring const & s);
int bytestring_casecmp(const_bytestring const & s1, const_charptr const & s2);
bool bytestring_caseprefix(const_bytestring const & s1, const_charptr const & s2);
double network_time();
namespace SSL {

class ContextSSL
{
public:
	ContextSSL(SSLAnalyzer * analyzer, SSLFlow * flow, FlowBuffer * flow_buffer);
	~ContextSSL();
	
	// Member access functions
	SSLAnalyzer * analyzer() const { return analyzer_; }
	SSLFlow * flow() const { return flow_; }
	FlowBuffer * flow_buffer() const { return flow_buffer_; }
	
protected:
	SSLAnalyzer * analyzer_;
	SSLFlow * flow_;
	FlowBuffer * flow_buffer_;
};


class uint24
{
public:
	uint24();
	~uint24();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data);
	
	// Member access functions
	uint8 byte1() const { return byte1_; }
	uint8 byte2() const { return byte2_; }
	uint8 byte3() const { return byte3_; }
	
protected:
	uint8 byte1_;
	uint8 byte2_;
	uint8 byte3_;
};


	class to_int {
	public:
		int operator()(uint24 * num) const
		{
		return (num->byte1() << 16) | (num->byte2() << 8) | num->byte3();
		}
	};


class ChangeCipherSpec
{
public:
	ChangeCipherSpec();
	~ChangeCipherSpec();
	int initial_buffer_length() const { return 0; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context);
	
	// Member access functions
	uint8 type() const { return type_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	uint8 type_;
	bool state_changed_;
	bool proc_;
	int buffering_state_;
};


class Alert
{
public:
	Alert();
	~Alert();
	int initial_buffer_length() const { return 0; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context);
	
	// Member access functions
	uint8 level() const { return level_; }
	uint8 description() const { return description_; }
	bool proc() const { return proc_; }
	
protected:
	uint8 level_;
	uint8 description_;
	bool proc_;
	int buffering_state_;
};


class V2Error
{
public:
	V2Error();
	~V2Error();
	int initial_buffer_length() const { return 0; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	uint16 error_code() const { return error_code_; }
	bool proc() const { return proc_; }
	
protected:
	uint16 error_code_;
	bool proc_;
	int buffering_state_;
};


class ApplicationData
{
public:
	ApplicationData();
	~ApplicationData();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool discard() const { return discard_; }
	bool proc() const { return proc_; }
	
protected:
	bool discard_;
	bool proc_;
};


class HelloRequest
{
public:
	HelloRequest();
	~HelloRequest();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool hr() const { return hr_; }
	
protected:
	bool hr_;
};


class ClientHello
{
public:
	ClientHello();
	~ClientHello();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	uint16 client_version() const { return client_version_; }
	uint32 gmt_unix_time() const { return gmt_unix_time_; }
	uint8 session_len() const { return session_len_; }
	vector<uint8> * session_id() const { return session_id_; }
	uint16 csuit_len() const { return csuit_len_; }
	vector<uint16> * csuits() const { return csuits_; }
	uint8 cmeth_len() const { return cmeth_len_; }
	vector<uint8> * cmeths() const { return cmeths_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	uint16 client_version_;
	uint32 gmt_unix_time_;
	const_bytestring random_bytes_;
	uint8 session_len_;
	vector<uint8> * session_id_;
	uint8 session_id__elem_;
	uint16 csuit_len_;
	vector<uint16> * csuits_;
	uint16 csuits__elem_;
	uint8 cmeth_len_;
	vector<uint8> * cmeths_;
	uint8 cmeths__elem_;
	bool state_changed_;
	bool proc_;
};


class V2ClientHello
{
public:
	V2ClientHello();
	~V2ClientHello();
	int initial_buffer_length() const { return 8; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	uint16 client_version() const { return client_version_; }
	uint16 csuit_len() const { return csuit_len_; }
	uint16 session_len() const { return session_len_; }
	uint16 chal_len() const { return chal_len_; }
	vector<uint24 *> * ciphers() const { return ciphers_; }
	vector<uint8> * session_id() const { return session_id_; }
	bytestring const & challenge() const { return challenge_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	uint16 client_version_;
	uint16 csuit_len_;
	uint16 session_len_;
	uint16 chal_len_;
	vector<uint24 *> * ciphers_;
	uint24 * ciphers__elem_;
	vector<uint8> * session_id_;
	uint8 session_id__elem_;
	bytestring challenge_;
	bool state_changed_;
	bool proc_;
	int buffering_state_;
};


class ServerHello
{
public:
	ServerHello();
	~ServerHello();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	uint16 server_version() const { return server_version_; }
	uint32 gmt_unix_time() const { return gmt_unix_time_; }
	uint8 session_len() const { return session_len_; }
	vector<uint8> * session_id() const { return session_id_; }
	vector<uint16> * cipher_suite() const { return cipher_suite_; }
	uint8 compression_method() const { return compression_method_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	uint16 server_version_;
	uint32 gmt_unix_time_;
	const_bytestring random_bytes_;
	uint8 session_len_;
	vector<uint8> * session_id_;
	uint8 session_id__elem_;
	vector<uint16> * cipher_suite_;
	uint16 cipher_suite__elem_;
	uint8 compression_method_;
	bool state_changed_;
	bool proc_;
};


class V2ServerHello
{
public:
	V2ServerHello();
	~V2ServerHello();
	int initial_buffer_length() const { return 10; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	uint8 session_id_hit() const { return session_id_hit_; }
	uint8 cert_type() const { return cert_type_; }
	uint16 server_version() const { return server_version_; }
	uint16 cert_len() const { return cert_len_; }
	uint16 ciph_len() const { return ciph_len_; }
	uint16 conn_id_len() const { return conn_id_len_; }
	bytestring const & cert_data() const { return cert_data_; }
	vector<uint24 *> * ciphers() const { return ciphers_; }
	bytestring const & conn_id_data() const { return conn_id_data_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	bool cert() const { return cert_; }
	
protected:
	uint8 session_id_hit_;
	uint8 cert_type_;
	uint16 server_version_;
	uint16 cert_len_;
	uint16 ciph_len_;
	uint16 conn_id_len_;
	bytestring cert_data_;
	vector<uint24 *> * ciphers_;
	uint24 * ciphers__elem_;
	bytestring conn_id_data_;
	bool state_changed_;
	bool proc_;
	bool cert_;
	int buffering_state_;
};


class X509Certificate
{
public:
	X509Certificate();
	~X509Certificate();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data);
	
	// Member access functions
	uint24 * length() const { return length_; }
	bytestring const & certificate() const { return certificate_; }
	
protected:
	uint24 * length_;
	bytestring certificate_;
};


class CertificateList
{
public:
	CertificateList();
	~CertificateList();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data);
	
	// Member access functions
	vector<X509Certificate *> * val() const { return val_; }
	int size() const	{ return val() ? val()->size() : 0; }
	X509Certificate * operator[](int index) const { BINPAC_ASSERT(val()); return (*val())[index]; }
	
protected:
	vector<X509Certificate *> * val_;
	X509Certificate * val__elem_;
};


class Certificate
{
public:
	Certificate();
	~Certificate();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	uint24 * length() const { return length_; }
	CertificateList * certificates() const { return certificates_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	uint24 * length_;
	CertificateList * certificates_;
	bool state_changed_;
	bool proc_;
};


class ServerKeyExchange
{
public:
	ServerKeyExchange();
	~ServerKeyExchange();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool state_changed() const { return state_changed_; }
	
protected:
	const_bytestring cont_;
	bool state_changed_;
};


class CertificateRequest
{
public:
	CertificateRequest();
	~CertificateRequest();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool state_changed() const { return state_changed_; }
	
protected:
	const_bytestring cont_;
	bool state_changed_;
};


class ServerHelloDone
{
public:
	ServerHelloDone();
	~ServerHelloDone();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool state_changed() const { return state_changed_; }
	
protected:
	bool state_changed_;
};


class ClientKeyExchange
{
public:
	ClientKeyExchange();
	~ClientKeyExchange();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool state_changed() const { return state_changed_; }
	
protected:
	const_bytestring cont_;
	bool state_changed_;
};


class V2ClientMasterKey
{
public:
	V2ClientMasterKey();
	~V2ClientMasterKey();
	int initial_buffer_length() const { return 9; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	uint24 * cipher_kind() const { return cipher_kind_; }
	uint16 cl_key_len() const { return cl_key_len_; }
	uint16 en_key_len() const { return en_key_len_; }
	uint16 key_arg_len() const { return key_arg_len_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	uint24 * cipher_kind_;
	uint16 cl_key_len_;
	uint16 en_key_len_;
	uint16 key_arg_len_;
	const_bytestring cl_key_data_;
	const_bytestring en_key_data_;
	const_bytestring key_arg_data_;
	bool state_changed_;
	bool proc_;
	int buffering_state_;
};


class CertificateVerify
{
public:
	CertificateVerify();
	~CertificateVerify();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool state_changed() const { return state_changed_; }
	
protected:
	const_bytestring cont_;
	bool state_changed_;
};


class UnknownHandshake
{
public:
	UnknownHandshake(uint8 msg_type);
	~UnknownHandshake();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	uint8 msg_type() const { return msg_type_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	const_bytestring cont_;
	uint8 msg_type_;
	bool state_changed_;
	bool proc_;
};


class Handshake
{
public:
	Handshake();
	~Handshake();
	int initial_buffer_length() const { return 4; }
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	uint8 msg_type() const { return msg_type_; }
	uint24 * length() const { return length_; }
	int body_case_index() const	{ return body_case_index_; }
	HelloRequest * hello_request() const
		{
		switch ( body_case_index() )
			{
			case 0:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:449:hello_request", body_case_index(), "HELLO_REQUEST");
				break;
			}
		return hello_request_;
		}
	ClientHello * client_hello() const
		{
		switch ( body_case_index() )
			{
			case 1:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:450:client_hello", body_case_index(), "CLIENT_HELLO");
				break;
			}
		return client_hello_;
		}
	ServerHello * server_hello() const
		{
		switch ( body_case_index() )
			{
			case 2:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:451:server_hello", body_case_index(), "SERVER_HELLO");
				break;
			}
		return server_hello_;
		}
	Certificate * certificate() const
		{
		switch ( body_case_index() )
			{
			case 11:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:452:certificate", body_case_index(), "CERTIFICATE");
				break;
			}
		return certificate_;
		}
	ServerKeyExchange * server_key_exchange() const
		{
		switch ( body_case_index() )
			{
			case 12:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:453:server_key_exchange", body_case_index(), "SERVER_KEY_EXCHANGE");
				break;
			}
		return server_key_exchange_;
		}
	CertificateRequest * certificate_request() const
		{
		switch ( body_case_index() )
			{
			case 13:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:454:certificate_request", body_case_index(), "CERTIFICATE_REQUEST");
				break;
			}
		return certificate_request_;
		}
	ServerHelloDone * server_hello_done() const
		{
		switch ( body_case_index() )
			{
			case 14:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:455:server_hello_done", body_case_index(), "SERVER_HELLO_DONE");
				break;
			}
		return server_hello_done_;
		}
	CertificateVerify * certificate_verify() const
		{
		switch ( body_case_index() )
			{
			case 15:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:456:certificate_verify", body_case_index(), "CERTIFICATE_VERIFY");
				break;
			}
		return certificate_verify_;
		}
	ClientKeyExchange * client_key_exchange() const
		{
		switch ( body_case_index() )
			{
			case 16:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:457:client_key_exchange", body_case_index(), "CLIENT_KEY_EXCHANGE");
				break;
			}
		return client_key_exchange_;
		}
	UnknownHandshake * unknown_handshake() const
		{
		return unknown_handshake_;
		}
	bool proc() const { return proc_; }
	
protected:
	uint8 msg_type_;
	uint24 * length_;
	int body_case_index_;
	HelloRequest * hello_request_;
	ClientHello * client_hello_;
	ServerHello * server_hello_;
	Certificate * certificate_;
	ServerKeyExchange * server_key_exchange_;
	CertificateRequest * certificate_request_;
	ServerHelloDone * server_hello_done_;
	CertificateVerify * certificate_verify_;
	ClientKeyExchange * client_key_exchange_;
	UnknownHandshake * unknown_handshake_;
	bool proc_;
	int buffering_state_;
};


class UnknownRecord
{
public:
	UnknownRecord();
	~UnknownRecord();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool discard() const { return discard_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	bool discard_;
	bool state_changed_;
	bool proc_;
};


class PlaintextRecord
{
public:
	PlaintextRecord();
	~PlaintextRecord();
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context, int t_byteorder);
	
	// Member access functions
	int val_case_index() const	{ return val_case_index_; }
	ChangeCipherSpec * ch_cipher() const
		{
		switch ( val_case_index() )
			{
			case 20:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:475:ch_cipher", val_case_index(), "CHANGE_CIPHER_SPEC");
				break;
			}
		return ch_cipher_;
		}
	Alert * alert() const
		{
		switch ( val_case_index() )
			{
			case 21:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:476:alert", val_case_index(), "ALERT");
				break;
			}
		return alert_;
		}
	Handshake * handshakes() const
		{
		switch ( val_case_index() )
			{
			case 22:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:477:handshakes", val_case_index(), "HANDSHAKE");
				break;
			}
		return handshakes_;
		}
	ApplicationData * app_data() const
		{
		switch ( val_case_index() )
			{
			case 23:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:478:app_data", val_case_index(), "APPLICATION_DATA");
				break;
			}
		return app_data_;
		}
	V2Error * v2_error() const
		{
		switch ( val_case_index() )
			{
			case 300:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:479:v2_error", val_case_index(), "V2_ERROR");
				break;
			}
		return v2_error_;
		}
	V2ClientHello * v2_client_hello() const
		{
		switch ( val_case_index() )
			{
			case 301:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:480:v2_client_hello", val_case_index(), "V2_CLIENT_HELLO");
				break;
			}
		return v2_client_hello_;
		}
	V2ClientMasterKey * v2_client_master_key() const
		{
		switch ( val_case_index() )
			{
			case 302:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:481:v2_client_master_key", val_case_index(), "V2_CLIENT_MASTER_KEY");
				break;
			}
		return v2_client_master_key_;
		}
	V2ServerHello * v2_server_hello() const
		{
		switch ( val_case_index() )
			{
			case 304:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:482:v2_server_hello", val_case_index(), "V2_SERVER_HELLO");
				break;
			}
		return v2_server_hello_;
		}
	UnknownRecord * unknown_record() const
		{
		switch ( val_case_index() )
			{
			case 400:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:483:unknown_record", val_case_index(), "UNKNOWN_OR_V2_ENCRYPTED");
				break;
			}
		return unknown_record_;
		}
	
protected:
	int val_case_index_;
	ChangeCipherSpec * ch_cipher_;
	Alert * alert_;
	Handshake * handshakes_;
	ApplicationData * app_data_;
	V2Error * v2_error_;
	V2ClientHello * v2_client_hello_;
	V2ClientMasterKey * v2_client_master_key_;
	V2ServerHello * v2_server_hello_;
	UnknownRecord * unknown_record_;
};


class CiphertextRecord
{
public:
	CiphertextRecord();
	~CiphertextRecord();
	int Parse(const_byteptr const t_begin_of_data, const_byteptr const t_end_of_data, ContextSSL * t_context);
	
	// Member access functions
	bool discard() const { return discard_; }
	bool state_changed() const { return state_changed_; }
	bool proc() const { return proc_; }
	
protected:
	bool discard_;
	bool state_changed_;
	bool proc_;
};


class SSLPDU
{
public:
	SSLPDU();
	~SSLPDU();
	bool ParseBuffer(flow_buffer_t t_flow_buffer, ContextSSL * t_context);
	
	// Member access functions
	int val_case_index() const	{ return val_case_index_; }
	CiphertextRecord * ciphertext() const
		{
		switch ( val_case_index() )
			{
			case 10:
			case 8:
			case 12:
			case 13:
				break;  // OK
			default:
				throw ExceptionInvalidCase("./ssl-protocol.pac:512:ciphertext", val_case_index(), "STATE_ABBREV_SERVER_ENCRYPTED, STATE_CLIENT_ENCRYPTED, STATE_COMM_ENCRYPTED, STATE_CONN_ESTABLISHED");
				break;
			}
		return ciphertext_;
		}
	PlaintextRecord * plaintext() const
		{
		return plaintext_;
		}
	int byteorder() const { return byteorder_; }
	bool consumed() const { return consumed_; }
	
protected:
	int val_case_index_;
	CiphertextRecord * ciphertext_;
	PlaintextRecord * plaintext_;
	int byteorder_;
	bool consumed_;
};


class SSLAnalyzer : public binpac::ConnectionAnalyzer
{
public:
	SSLAnalyzer();
	~SSLAnalyzer();
	
	// Member access functions
	SSLFlow * upflow() const { return upflow_; }
	SSLFlow * downflow() const { return downflow_; }
	
	void NewData(bool is_orig, const_byteptr begin, const_byteptr end);
	void NewGap(bool is_orig, int gap_length);
	void FlowEOF(bool is_orig);
	
	// Functions
	int current_record_type();
	int current_record_version();
	int current_record_length();
	bool current_record_is_orig();
	bool next_record(const_bytestring const & rec, int type, int version, bool is_orig);
	int state();
	int old_state();
	bool transition(AnalyzerState olds, AnalyzerState news, bool is_orig);
	bool lost_track();
	bool hello_requested();
	bool set_hello_requested(bool val);
	Analyzer * bro_analyzer();
	void set_bro_analyzer(Analyzer * a);
	bool check_cipher(int cipher);
	void certificate_error(int err_num);
	bool proc_change_cipher_spec(ChangeCipherSpec * msg);
	bool proc_application_data(ApplicationData * msg);
	bool proc_alert(int level, int description);
	bool proc_client_hello(uint16 version, vector<uint8> * session_id, vector<int> * csuits);
	bool proc_server_hello(uint16 version, vector<uint8> * session_id, vector<int> * ciphers, int v2_sess_hit);
	bool proc_certificate(vector<bytestring> * certificates);
	bool proc_v2_certificate(bytestring const & cert);
	bool proc_v3_certificate(CertificateList * cl);
	bool proc_v2_client_master_key(int cipher);
	bool proc_unknown_handshake(int msg_type);
	bool proc_handshake(Handshake * msg);
	bool proc_unknown_record(UnknownRecord * msg);
	bool proc_ciphertext_record(CiphertextRecord * msg);
	
protected:
	
	// Additional members

		int current_record_type_;
		int current_record_version_;
		int current_record_length_;
		bool current_record_is_orig_;
		int state_;
		int old_state_;
		bool hello_requested_;
	

		Analyzer* bro_analyzer_;

		vector<uint8>* client_session_id_;
		vector<int>* advertised_ciphers_;
		int version_;
		int cipher_;
	
	SSLFlow * upflow_;
	SSLFlow * downflow_;
};


class SSLFlow : public binpac::FlowAnalyzer
{
public:
	SSLFlow(SSLAnalyzer * connection, bool is_orig);
	~SSLFlow();
	
	// Member access functions
	SSLAnalyzer * connection() const { return connection_; }
	bool is_orig() const { return is_orig_; }
	
	void NewData(const_byteptr t_begin_of_data, const_byteptr t_end_of_data);
	void NewGap(int gap_length);
	void FlowEOF();
	
	// Functions
	bool discard_data();
	bool data_available();
	bool consume_data();
	
protected:
	SSLPDU * dataunit_;
	ContextSSL * context_;
	FlowBuffer * flow_buffer_;
	SSLAnalyzer * connection_;
	bool is_orig_;
};


	class extract_certs {
	public:
		bytestring const& operator() (X509Certificate* cert) const
			{
			return cert->certificate();
			}
	};

#ifdef USE_OPENSSL
	void free_X509(void *);
	X509* d2i_X509_binpac(X509** px, const uint8** in, int len);
#endif
	
TableVal * to_table_val(vector<uint8> * data);
bool version_ok(uint16 vers);
vector<int> * convert_ciphers_uint24(vector<uint24 *> * ciph);
vector<int> * convert_ciphers_uint16(vector<uint16> * ciph);
} // namespace SSL
}  // namespace binpac
#endif /* __ssl_pac_h */
